@@ -42,8 +42,6 @@
 #define BOGUS_URI "file:///bogus/"
 #define BOGUS_URI_LEN 14
 
-static void        yelp_view_init                 (YelpView           *view);
-static void        yelp_view_class_init           (YelpViewClass      *klass);
 static void        yelp_view_dispose              (GObject            *object);
 static void        yelp_view_finalize             (GObject            *object);
 static void        yelp_view_get_property         (GObject            *object,
@@ -150,7 +148,7 @@ enum {
 };
 static gint signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (YelpView, yelp_view, WEBKIT_TYPE_WEB_VIEW);
+G_DEFINE_TYPE (YelpView, yelp_view, WEBKIT_TYPE_WEB_VIEW)
 #define GET_PRIV(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), YELP_TYPE_VIEW, YelpViewPrivate))
 
 static WebKitWebSettings *websettings;
@@ -243,7 +241,6 @@ enum {
 static void
 yelp_view_init (YelpView *view)
 {
-    GtkAction *action;
     YelpViewPrivate *priv = GET_PRIV (view);
 
     g_object_set (view, "settings", websettings, NULL);
@@ -775,7 +772,7 @@ view_install_uri (YelpView    *view,
                   const gchar *uri)
 {
     GDBusConnection *connection;
-    GError *error;
+    GError *error = NULL;
     gboolean help = FALSE, ghelp = FALSE;
     GVariantBuilder *strv;
     YelpInstallInfo *info;
@@ -784,7 +781,7 @@ view_install_uri (YelpView    *view,
     GtkWidget *gtkwin;
     GdkWindow *gdkwin;
     /* do not free */
-    gchar *pkg, *confirm_search;
+    const gchar *pkg, *confirm_search;
 
     if (g_str_has_prefix (uri, "install-help:")) {
         help = TRUE;
@@ -798,7 +795,7 @@ view_install_uri (YelpView    *view,
         pkg = (gchar *) uri + 8;
     }
 
-    connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
+    connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
     if (connection == NULL) {
         g_warning ("Unable to connect to dbus: %s", error->message);
         g_error_free (error);
@@ -899,7 +896,7 @@ view_scrolled (GtkAdjustment *adjustment,
         return;
     if (adjustment == priv->vadjustment)
         ((YelpBackEntry *) priv->back_cur->data)->vadj = gtk_adjustment_get_value (adjustment);
-    else if (adjustment = priv->hadjustment)
+    else if (adjustment == priv->hadjustment)
         ((YelpBackEntry *) priv->back_cur->data)->hadj = gtk_adjustment_get_value (adjustment);
 }
 
@@ -1157,26 +1154,26 @@ popup_save_code (GtkMenuItem *item,
                                                     NULL,
                                                     &error);
         if (stream == NULL) {
-            GtkWidget *dialog = gtk_message_dialog_new (gtk_widget_get_visible (window) ? GTK_WINDOW (window) : NULL,
-                                                        GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                        GTK_MESSAGE_ERROR,
-                                                        GTK_BUTTONS_OK,
-                                                        "%s", error->message);
-            gtk_dialog_run (GTK_DIALOG (dialog));
-            gtk_widget_destroy (dialog);
+            GtkWidget *dlg = gtk_message_dialog_new (gtk_widget_get_visible (window) ? GTK_WINDOW (window) : NULL,
+                                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                     GTK_MESSAGE_ERROR,
+                                                     GTK_BUTTONS_OK,
+                                                     "%s", error->message);
+            gtk_dialog_run (GTK_DIALOG (dlg));
+            gtk_widget_destroy (dlg);
             g_error_free (error);
         }
         else {
             /* FIXME: we should do this async */
             GDataOutputStream *datastream = g_data_output_stream_new (G_OUTPUT_STREAM (stream));
             if (!g_data_output_stream_put_string (datastream, priv->popup_code_text, NULL, &error)) {
-                GtkWidget *dialog = gtk_message_dialog_new (gtk_widget_get_visible (window) ? GTK_WINDOW (window) : NULL,
-                                                            GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                            GTK_MESSAGE_ERROR,
-                                                            GTK_BUTTONS_OK,
-                                                            "%s", error->message);
-                gtk_dialog_run (GTK_DIALOG (dialog));
-                gtk_widget_destroy (dialog);
+                GtkWidget *dlg = gtk_message_dialog_new (gtk_widget_get_visible (window) ? GTK_WINDOW (window) : NULL,
+                                                         GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                         GTK_MESSAGE_ERROR,
+                                                         GTK_BUTTONS_OK,
+                                                         "%s", error->message);
+                gtk_dialog_run (GTK_DIALOG (dlg));
+                gtk_widget_destroy (dlg);
                 g_error_free (error);
             }
             g_object_unref (datastream);
@@ -1316,7 +1313,7 @@ view_populate_popup (YelpView *view,
             gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
         }
         else {
-            GSList *cur;
+            GSList *l;
 
             item = gtk_menu_item_new_with_mnemonic (_("_Open Link"));
             g_signal_connect (item, "activate",
@@ -1337,9 +1334,9 @@ view_populate_popup (YelpView *view,
                 gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
             }
 
-            for (cur = priv->link_actions; cur != NULL; cur = cur->next) {
+            for (l = priv->link_actions; l != NULL; l = l->next) {
                 gboolean add;
-                YelpActionEntry *entry = (YelpActionEntry *) cur->data;
+                YelpActionEntry *entry = (YelpActionEntry *) l->data;
                 if (entry->func == NULL)
                     add = TRUE;
                 else
@@ -1488,7 +1485,6 @@ view_resource_request (WebKitWebView         *view,
 {
     YelpViewPrivate *priv = GET_PRIV (view);
     const gchar *requri = webkit_network_request_get_uri (request);
-    gchar last;
     gchar *newpath;
 
     if (!g_str_has_prefix (requri, BOGUS_URI))
@@ -1548,7 +1544,32 @@ view_document_loaded (WebKitWebView   *view,
 static void
 view_print_action (GAction *action, GVariant *parameter, YelpView *view)
 {
-    webkit_web_frame_print (webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (view)));
+    WebKitWebFrame *main_frame;
+    GtkPrintOperation *operation;
+    GtkPrintSettings *settings;
+    GError *error = NULL;
+    YelpViewPrivate *priv = GET_PRIV (view);
+
+    main_frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (view));
+
+    operation = gtk_print_operation_new ();
+    settings = gtk_print_settings_new ();
+    gtk_print_settings_set (settings,
+                            GTK_PRINT_SETTINGS_OUTPUT_BASENAME,
+                            priv->page_title);
+    gtk_print_operation_set_print_settings (operation, settings);
+
+    webkit_web_frame_print_full (main_frame, operation,
+                                 GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
+                                 &error);
+
+    if (error) {
+        g_warning ("Error showing print dialog: %s", error->message);
+        g_error_free (error);
+    }
+
+    g_object_unref (operation);
+    g_object_unref (settings);
 }
 
 static void
@@ -1676,52 +1697,53 @@ view_load_page (YelpView *view)
     g_free (page_id);
 }
 
+#define FORMAT_ERRORPAGE \
+    "<html><head>" \
+    "<style type='text/css'>" \
+    "body {" \
+    " margin: 1em;" \
+    " color: %s;" \
+    " background-color: %s;" \
+    " }\n" \
+    "p { margin: 1em 0 0 0; }\n" \
+    "div.note {" \
+    " padding: 6px;" \
+    " border-color: %s;" \
+    " border-top: solid 1px;" \
+    " border-bottom: solid 1px;" \
+    " background-color: %s;" \
+    " }\n" \
+    "div.note div.inner {" \
+    " margin: 0; padding: 0;" \
+    " background-image: url(%s);" \
+    " background-position: %s top;" \
+    " background-repeat: no-repeat;" \
+    " min-height: %ipx;" \
+    " }\n" \
+    "div.note div.contents {" \
+    " margin-%s: %ipx;" \
+    " }\n" \
+    "div.note div.title {" \
+    " margin-%s: %ipx;" \
+    " margin-bottom: 0.2em;" \
+    " font-weight: bold;" \
+    " color: %s;" \
+    " }\n" \
+    "a { color: %s; text-decoration: none; }\n" \
+    "</style>" \
+    "</head><body>" \
+    "<div class='note'><div class='inner'>" \
+    "%s<div class='contents'>%s%s</div>" \
+    "</div></div>" \
+    "</body></html>"
+
 static void
 view_show_error_page (YelpView *view,
                       GError   *error)
 {
     YelpViewPrivate *priv = GET_PRIV (view);
-    static const gchar *errorpage =
-        "<html><head>"
-        "<style type='text/css'>"
-        "body {"
-        " margin: 1em;"
-        " color: %s;"
-        " background-color: %s;"
-        " }\n"
-        "p { margin: 1em 0 0 0; }\n"
-        "div.note {"
-        " padding: 6px;"
-        " border-color: %s;"
-        " border-top: solid 1px;"
-        " border-bottom: solid 1px;"
-        " background-color: %s;"
-        " }\n"
-        "div.note div.inner {"
-        " margin: 0; padding: 0;"
-        " background-image: url(%s);"
-        " background-position: %s top;"
-        " background-repeat: no-repeat;"
-        " min-height: %ipx;"
-        " }\n"
-        "div.note div.contents {"
-        " margin-%s: %ipx;"
-        " }\n"
-        "div.note div.title {"
-        " margin-%s: %ipx;"
-        " margin-bottom: 0.2em;"
-        " font-weight: bold;"
-        " color: %s;"
-        " }\n"
-        "a { color: %s; text-decoration: none; }\n"
-        "</style>"
-        "</head><body>"
-        "<div class='note'><div class='inner'>"
-        "%s<div class='contents'>%s%s</div>"
-        "</div></div>"
-        "</body></html>";
     YelpSettings *settings = yelp_settings_get_default ();
-    gchar *page, *title = NULL, *link = NULL, *title_m, *content_beg, *content_end;
+    gchar *page, *title = NULL, *title_m, *content_beg, *content_end;
     gchar *textcolor, *bgcolor, *noteborder, *notebg, *titlecolor, *noteicon, *linkcolor;
     gint iconsize;
     GParamSpec *spec;
@@ -1753,7 +1775,7 @@ view_show_error_page (YelpView *view,
     if (doc404) {
         gchar *struri = yelp_uri_get_document_uri (priv->uri);
         /* do not free */
-        gchar *pkg = NULL, *scheme = NULL;
+        const gchar *pkg = NULL, *scheme = NULL;
         if (g_str_has_prefix (struri, "help:")) {
             scheme = "help";
             pkg = struri + 5;
@@ -1778,7 +1800,7 @@ view_show_error_page (YelpView *view,
     noteicon = yelp_settings_get_icon (settings, YELP_SETTINGS_ICON_WARNING);
     iconsize = yelp_settings_get_icon_size (settings) + 6;
 
-    page = g_strdup_printf (errorpage,
+    page = g_strdup_printf (FORMAT_ERRORPAGE,
                             textcolor, bgcolor, noteborder, notebg, noteicon,
                             left, iconsize, left, iconsize, left, iconsize,
                             titlecolor, linkcolor, title_m, content_beg,
@@ -1884,20 +1906,22 @@ uri_resolved (YelpUri  *uri,
               YelpView *view)
 {
     YelpViewPrivate *priv = GET_PRIV (view);
+    YelpUriDocumentType doctype;
     YelpDocument *document;
     YelpBackEntry *back;
     GError *error = NULL;
     gchar *struri;
     GParamSpec *spec;
 
-    if (yelp_uri_get_document_type (uri) != YELP_URI_DOCUMENT_TYPE_EXTERNAL) {
+    doctype = yelp_uri_get_document_type (uri);
+
+    if (doctype != YELP_URI_DOCUMENT_TYPE_EXTERNAL) {
         g_object_ref (uri);
         view_clear_load (view);
         priv->uri = uri;
     }
 
-    switch (yelp_uri_get_document_type (uri)) {
-    case YELP_URI_DOCUMENT_TYPE_EXTERNAL:
+    if (doctype == YELP_URI_DOCUMENT_TYPE_EXTERNAL) {
         g_object_set (view, "state", priv->prevstate, NULL);
         struri = yelp_uri_get_canonical_uri (uri);
         if (g_str_has_prefix (struri, "install:") ||
@@ -1911,7 +1935,8 @@ uri_resolved (YelpUri  *uri,
         }
         g_free (struri);
         return;
-    case YELP_URI_DOCUMENT_TYPE_NOT_FOUND:
+    }
+    else if (doctype == YELP_URI_DOCUMENT_TYPE_NOT_FOUND) {
         struri = yelp_uri_get_canonical_uri (uri);
         if (struri != NULL) {
             error = g_error_new (YELP_ERROR, YELP_ERROR_NOT_FOUND,
@@ -1923,16 +1948,13 @@ uri_resolved (YelpUri  *uri,
             error = g_error_new (YELP_ERROR, YELP_ERROR_NOT_FOUND,
                                  _("The URI does not point to a valid page."));
         }
-        break;
-    case YELP_URI_DOCUMENT_TYPE_ERROR:
+    }
+    else if (doctype == YELP_URI_DOCUMENT_TYPE_ERROR) {
         struri = yelp_uri_get_canonical_uri (uri);
         error = g_error_new (YELP_ERROR, YELP_ERROR_PROCESSING,
                              _("The URI ‘%s’ could not be parsed."),
                              struri);
         g_free (struri);
-        break;
-    default:
-        break;
     }
 
     if (error == NULL) {
@@ -2026,7 +2048,6 @@ document_callback (YelpDocument       *document,
             g_free (real_id);
         }
         else {
-            GParamSpec *spec;
             g_free (priv->page_id);
             priv->page_id = real_id;
             spec = g_object_class_find_property ((GObjectClass *) YELP_VIEW_GET_CLASS (view),
